SpriteAnimator = class("SpriteAnimator")

function SpriteAnimator:created()
    -- you can accept and set parameters here
    
    self.entity.getSpriteWorldSize = function() 
        return (vec2(select(3, self.entity.sprite:rect()), select(4, self.entity.sprite:rect())) / scene.main.pixelsPerUnit) * vec2(self.entity.scale.x, self.entity.scale.y)
    end
    
    self.animations = {}
    self.frameNumber = 1
    
    self.timeOffset = time.elapsed
    self.repeatNum = nil
    
    self.currentAnimation = nil
    self.currentAnimationName = nil
    self.playingAnimation = false
    self.hasEnded = false
    self.isLocked = false
    self.gotoframe = nil
    self.smooth = true
    self.canEnd = false
end

function SpriteAnimator:add(name, animation)
    self.animations[name] = animation
end

function SpriteAnimator:getFrameSize()
    if self.currentAnimation then
        return vec2(select(3, self.entity.sprite:rect()), select(4, self.entity.sprite:rect()))
    end
end

function SpriteAnimator:getFrameWorldSize()
    if self.currentAnimation then
        return self.entity:getSpriteWorldSize()
    end
end

function SpriteAnimator:pause()
    self.playingAnimation = false
end

function SpriteAnimator:resume()
    self.timeOffset = time.elapsed - ((self.frameNumber - 1.0) / self.currentAnimation.fps)
    
    self:checkGoto()
    self.playingAnimation = true
end

function SpriteAnimator:goToFrame(frameNum)
    self.gotoframe = frameNum
end 

function SpriteAnimator:lockAnimation()
    self.isLocked = true
end

function SpriteAnimator:releaseAnimation()
    self.isLocked = false
end

function SpriteAnimator:getAnimation(name)
    return self.animations[name]
end

function SpriteAnimator:getCurrentAnimation()
    return self.currentAnimation
end

function SpriteAnimator:play(name, num)
    if not self.isLocked and self.currentAnimationName ~= name then
        self.timeOffset = time.elapsed
        self.playingAnimation = true
        self.currentAnimation = self.animations[name]
        self.currentAnimationName = name
        self.entity.sprite = self.currentAnimation.frames[1]
        
        self.repeatNum = num
        
    end
end

function SpriteAnimator:update()
    if ((self.repeatNum == -1 or self.repeatNum ~= 0) and self.playingAnimation)  then
        self.currentAnimation.sheetImg.smooth = self.smooth
        self:checkGoto()
        
        if self.hasEnded then
            self.hasEnded = false
            self.canEnd = false
        elseif self.frameNumber == 1 and self.canEnd then
            self.hasEnded = true
        end
        
        if self.frameNumber > 1 then
            self.canEnd = true
        end
        
        self.frameNumber = (math.floor((time.elapsed - self.timeOffset) * self.currentAnimation.fps) % #self.currentAnimation.frames) + 1
        self.entity.sprite = self.currentAnimation.frames[self.frameNumber]
        
        if self.hasEnded and self.repeatNum then
             self.repeatNum = self.repeatNum - 1
        end
    end
end

function SpriteAnimator:checkGoto()
    if self.gotoframe then
        self.timeOffset = time.elapsed - ((self.gotoframe - 1.0) / self.currentAnimation.fps)
        self.frameNumber = (math.floor((time.elapsed - self.timeOffset) * self.currentAnimation.fps) % #self.currentAnimation.frames) + 1
        self.gotoframe = nil
    end
end
